//
// JMarsApplet.java
//	Copyright 2003 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

//package Akk.Astro;

import java.awt.*;
import java.applet.*;
//import java.util.Date;
import java.net.URL;
import java.awt.event.*;
import java.util.*;

public class JMarsApplet extends Applet
    implements ActionListener, ItemListener
{
     // Copies of the UI components we maintain:
    MarsCanvas marsCanvas;
    MarsFlipPanel flip;
    StarDate curDate;
    boolean northUp;
    boolean eastRight;

    String mapNormal;
    String mapEWFlip;
    String mapNSRot;
    String mapNSEW;
    String mapBW;

    static final String NormalBtnStr = new String("Normal");
    static final String EWBtnStr = new String("EW Flip");
    static final String NSBtnStr = new String("NS Rot");
    static final String NSEWBtnStr = new String("Flip+Rot");
    static final String BWBtnStr = new String("B&W");

    Mars mars;
    Image mapImage;
    Image globeImage;
    SphereMap sph;

    class MarsFlipPanel extends Panel
    {
        Button NormalBtn;
        Button EWBtn;
        Button NSBtn;
        Button NSEWBtn;
        Button BWBtn;
        MarsFlipPanel(EventListener listener)
        {
            setLayout(new FlowLayout(FlowLayout.LEFT));
            Label labl = new Label("View:");
            this.add(labl);
            if (mapNormal != null) {
                NormalBtn = new Button(NormalBtnStr);
                NormalBtn.addActionListener((ActionListener)listener);
                this.add(NormalBtn);
            }
            else NormalBtn = null;
            if (mapEWFlip != null) {
                EWBtn = new Button(EWBtnStr);
                EWBtn.addActionListener((ActionListener)listener);
                this.add(EWBtn);
            }
            else EWBtn = null;
            if (mapNSRot != null) {
                NSBtn = new Button(NSBtnStr);
                NSBtn.addActionListener((ActionListener)listener);
                this.add(NSBtn);
            }
            else NSBtn = null;
            if (mapNSEW != null) {
                NSEWBtn = new Button(NSEWBtnStr);
                NSEWBtn.addActionListener((ActionListener)listener);
                this.add(NSEWBtn);
            }
            else NSEWBtn = null;
            if (mapBW != null) {
                BWBtn = new Button(BWBtnStr);
                BWBtn.addActionListener((ActionListener)listener);
                this.add(BWBtn);
            }
            else BWBtn = null;

            // Disable the first one we made, and hope that's right:
            if (NormalBtn != null)
                disable(NormalBtnStr);
            else if (EWBtn != null)
                disable(EWBtnStr);
            else if (NSBtn != null)
                disable(NSBtnStr);
            else if (NSEWBtn != null)
                disable(NSEWBtnStr);
        }

        // Grey out one button, and enable the others
        void disable(String which) {
            if (NormalBtn != null)
                NormalBtn.setEnabled(which != NormalBtnStr);
            if (EWBtn != null)
                EWBtn.setEnabled(which != EWBtnStr);
            if (NSBtn != null)
                NSBtn.setEnabled(which != NSBtnStr);
            if (NSEWBtn != null)
                NSEWBtn.setEnabled(which != NSEWBtnStr);
            if (BWBtn != null)
                BWBtn.setEnabled(which != BWBtnStr);
        }
    }

    class MarsCanvas extends Canvas
    {
        JMarsApplet app;

        public MarsCanvas(Mars m, Image mi, JMarsApplet jma) {
            mars = m;
            mapImage = mi;
            app = jma;
            sph = new SphereMap();
        }

        public void paint(Graphics g)
        {
            int startX = 5;
            int imStartY = 5;
            int textStartY = -8;
            int textHeight = 16;  // should get from the font somehow!
            int direcStartX = (int)(globeImage.getWidth(app) * .65);
            int direcTopY = 10;
            int direcBottomY = globeImage.getHeight(app)+10;
            //setBackground(Color.white);
            setForeground(Color.black);

            g.drawImage(globeImage, startX, imStartY, app);

            g.drawString("Size: " + new Integer((int)mars.getSize()),
                         startX,
                         globeImage.getHeight(app) + textStartY);
            g.drawString("CM: " + new Integer((int)(mars.getCM()*180/Math.PI)),
                         startX,
                         globeImage.getHeight(app) + textStartY + textHeight);
            g.drawString(app.curDate.toString(),
                         startX,
                         globeImage.getHeight(app)
                         + textStartY + textHeight*2);

            g.drawString(northUp ? "N" : "S", direcStartX, direcTopY);
            g.drawString(northUp ? "S" : "N", direcStartX, direcBottomY);
            g.drawString(eastRight ? "E" : "W", globeImage.getWidth(app)+7, globeImage.getHeight(app)/2);
        }
    }

    public String getAppletInfo() {
	return "JMars.\nBy Akkana Peck";
    }

    public String[][] getParameterInfo() {
	String[][] info = {
	    { "time", "double",
	      "Time of chart (optional, defaults to current)" },
	    { "mapImageNormal", "String", "Mars map image to use" },
	    { "mapImageEWFlip", "String", "Mars map image to use for EW flip" },
	    { "mapImageNSRot", "String", "Mars map image to use for NS rot" },
	    { "mapImageNSEW", "String", "Mars map image to use when flipped and inverted" },
	    { "mapImageBW", "String", "Mars map image to use for black&white" },
   	};
	return info;
    }

    public void init()
    {
	//
	// Get the parameters:
	//
    	String time = getParameter("time");
	if (time != null) {
            try {
                curDate = new StarDate(time);
            }
            catch (IllegalArgumentException e) {
                System.out.println("Illegal date format: " + time);
                curDate = new StarDate();
            }
        } else {
            curDate = new StarDate();
        }

	mapNormal = getParameter("mapImageNormal");
	mapEWFlip = getParameter("mapImageEWFlip");
	mapNSRot = getParameter("mapImageNSRot");
	mapNSEW = getParameter("mapImageNSEW");
	mapBW = getParameter("mapImageBW");
        String s = null;
        if (mapNormal != null) {
            s = mapNormal;
            northUp = true;
            eastRight = true;
        }
        else if (mapEWFlip != null) {
            //System.out.println("east-west flip\n");
            s = mapEWFlip;
            northUp = true;
            eastRight = false;
        }
        else if (mapNSRot != null) {
            //System.out.println("north-south rot\n");
            s = mapNSRot;
            northUp = false;
            eastRight = false;
        }
        else if (mapNSEW != null) {
            //System.out.println("north-south and east-west\n");
            s = mapNSEW;
            northUp = false;
            eastRight = true;
        }
        else if (mapBW != null) {
            //System.out.println("black-and-white\n");
            s = mapBW;
            northUp = false;
            eastRight = true;
        }
        // else we're in deep doo-doo because we don't have a map at all
	if (s != null) {
            URL codebase = getCodeBase();
            mapImage = getImage(codebase, s);
            //System.out.println("Loaded image " + s + "\n");
        }

	s = getParameter("northUp");
	if (s != null) {
            northUp = Boolean.valueOf(s).booleanValue();
        } else {
            northUp = true;
        }

        mars = new Mars();

	//
	// Create the basic UI
	//

        //
	// Set some layout info:
	//
	setLayout(new BorderLayout(0, 1));

        flip = new MarsFlipPanel(this);
	add("North", flip);

	marsCanvas = new MarsCanvas(mars, mapImage, this);
	add("Center", marsCanvas);

        DatePanel panel = new DatePanel(this);
        //panel.addActionListener(this);
        add("South", panel);

        recalcMars();
    }

    public void recalcMars()
    {
        MediaTracker mt = new MediaTracker(this);
        mt.addImage(mapImage, 0);
        try {
            mt.waitForID(0);
        } catch (InterruptedException e) {
            System.out.println("Media Tracker Error " + e);
            return;
        }
        mars.CalcCM(curDate.getJulianDate());
        globeImage = sph.mapImageToSphere(mapImage, this,
                                       (int)(mars.getCM()*180/Math.PI),
                                       (int)(mars.getLat()*180/Math.PI),
                                       northUp, eastRight);
    }

    public boolean handleEvent(Event event)
    {
	return super.handleEvent(event);
    }

    public void actionPerformed(ActionEvent e)
    {
        if (e.getActionCommand().equals(new String("Date...")))
        {
            Container container = this.getParent();
            while (! (container instanceof Frame))
                container = container.getParent();
            CalenDialog d = new CalenDialog((Frame)container, this);
            d.show();
        }
        else if (e.getActionCommand().equals(new String("Now")))
        {
            curDate = new StarDate();
            recalcMars();
            repaint();
        }
        else if (e.getActionCommand().equals(new String("H+")))
        {
            curDate.setTime(curDate.getTime() + 1000*60*60);
            recalcMars();
            repaint();
        }
        else if (e.getActionCommand().equals(new String("H-")))
        {
            curDate.setTime(curDate.getTime() - 1000*60*60);
            recalcMars();
            repaint();
        }
        else if (e.getActionCommand().equals(new String("D+")))
        {
            curDate.addDays(1);
            recalcMars();
            repaint();
        }
        else if (e.getActionCommand().equals(new String("D-")))
        {
            curDate.addDays(-1);
            recalcMars();
            repaint();
        }
        else if (e.getActionCommand().equals(new String("dateChanged")))
        {
            StarCalendar cal = (StarCalendar)e.getSource();
            curDate = cal.getDate();
            recalcMars();
            repaint();
        }

        // From flip controls
        else if (e.getActionCommand().equals(NormalBtnStr)
                 && mapNormal != null)
        {
            URL codebase = getCodeBase();
            mapImage = getImage(codebase, mapNormal);
            northUp = true;
            eastRight = true;
            flip.disable(NormalBtnStr);
            recalcMars();
            repaint();
        }
        else if (e.getActionCommand().equals(EWBtnStr) && mapEWFlip != null)
        {
            URL codebase = getCodeBase();
            mapImage = getImage(codebase, mapEWFlip);
            northUp = true;
            eastRight = false;
            flip.disable(EWBtnStr);
            recalcMars();
            repaint();
        }
        else if (e.getActionCommand().equals(NSBtnStr) && mapNSRot != null)
        {
            URL codebase = getCodeBase();
            mapImage = getImage(codebase, mapNSRot);
            northUp = false;
            eastRight = false;
            flip.disable(NSBtnStr);
            recalcMars();
            repaint();
        }
        else if (e.getActionCommand().equals(NSEWBtnStr) && mapNSEW != null)
        {
            URL codebase = getCodeBase();
            mapImage = getImage(codebase, mapNSEW);
            northUp = false;
            eastRight = true;
            flip.disable(NSEWBtnStr);
            recalcMars();
            repaint();
        }
        else if (e.getActionCommand().equals(BWBtnStr) && mapBW != null)
        {
            URL codebase = getCodeBase();
            mapImage = getImage(codebase, mapBW);
            // Hack: right now we only have one B&W image, so hardwire it.
            northUp = false;
            eastRight = false;
            flip.disable(BWBtnStr);
            recalcMars();
            repaint();
        }
    }

    public void itemStateChanged(ItemEvent e)
    {
        System.out.println("item state changed: " + e.getItem().toString());
    }

    public void paint(Graphics g)
    {
    	int width = size().width;
        int height = size().height;

        marsCanvas.repaint();
    }
}

